import os
from typing import Dict, List, Set
from collections import defaultdict

import toml

from ..vitae import load_vitae
from ..vitae.tech_skills import get_skills_data
from .schema import AggregateContent, SkillCount, TopicCount

CURDIR = f"{os.path.dirname(__file__)}{os.path.sep}"


def load_aggregate(
    agg_path: str,
    vitae_paths: List[str],
) -> AggregateContent:
    assert str(agg_path).endswith(".toml"), (
        "must be TOML files with .toml extension.")
    assert all(path.endswith(".toml") for path in vitae_paths), (
        "must be TOML files with .toml extension.")

    with open(agg_path) as src:
        aggr = AggregateContent(**toml.load(src))

    # transpose technical skill: [{s1}, {s1, s2}] -> {s1:2, s2:1}
    vitaes = [load_vitae(path) for path in vitae_paths]
    skill_count: Dict[str, int] = defaultdict(int)
    topic_count: Dict[str, int] = defaultdict(int)
    universities: Set[str] = set()
    nationalities: Set[str] = set()
    languages_spoken: Set[str] = set()
    for vitae in vitaes:
        for skill in vitae.technical_skill:
            for value in skill.values:
                skill_count[value] += 1

        for topic in set(edu.topic for edu in vitae.education):
            if topic:
                topic_count[topic] += 1

        aggr.num_doctors += any(edu.degree in ("PhD", "Doctor Scient")
                                for edu in vitae.education)
        universities.update(edu.university for edu in vitae.education)
        if vitae.nationality:
            nationalities.add(vitae.nationality)
        languages_spoken.update(lan.language for lan in vitae.language_skill)

    skills = get_skills_data()["skills"]
    unknown_skills = set(aggr.technical_skills).difference(skills)
    assert not unknown_skills, f"unallowed skills: {unknown_skills}"

    aggr.skill_count = [
        SkillCount(value=value, count=skill_count[value])
        for value in sorted(skill_count.keys())
        if value in aggr.technical_skills
    ]
    aggr.topic_count = [
        TopicCount(value=value, count=topic_count[value])
        for value in sorted(topic_count.keys())
        if value in aggr.topics
    ]
    aggr.num_employees = len(vitaes)
    aggr.num_nationalities = len(nationalities)
    aggr.num_languages_spoken = len(languages_spoken)
    aggr.num_universities_attended = len(universities)

    return aggr



"""Tool for migrating from old yaml to new one."""
import yaml
import toml


def convert_yaml_to_toml(source: str, target: str) -> None:
    """Convert from old to new yaml format."""

    with open(source) as src:
        content = yaml.safe_load(src)

    with open(target, "w") as dst:
        for name in content["Basic"]:
            toml.dump({name.lower(): content["Basic"][name]}, dst)

        toml.dump({"summary": content["Summary"].strip()}, dst)
        dst.write("\n")

        toml.dump({"technical_skill": [value for _, values in content.get("Skills", {}).items()
                                       for value in values]}, dst)
        dst.write("\n")

        toml.dump({"language_skill": [dict(language=language, proficiency=proficiency)
                                      for language, proficiency in content.get("Languages", {}).items()]}, dst)
        toml.dump({"personal_skill": [dict(title=title, description=description.strip())
                                      for title, description in content.get("SelectedSkills", {}).items()]}, dst)
        toml.dump({"hobby": [dict(title=title, values=values)
                             for title, values in content.get("Interests", {}).items()]}, dst)
        toml.dump({"education": [dict(year=year[-1], description=description.strip())
                                 for *year, description in content.get("Education", {})]}, dst)
        toml.dump({"work": [
            (dict(description=description.strip(), start=period[0]) if len(period) == 1 or not period[1] else
             dict(description=description.strip(), start=period[0], end=period[1]))
                                 for *period, description in content.get("Work", {})]}, dst)

        out = []
        for tag, project in content.get("Projects", {}).items():
            out.append({key.lower(): val for key, val in project.items()})
            out[-1]["tag"] = tag[1:]
        toml.dump({"project": out}, dst)

        out = []
        for tag, project in content.get("Publications", {}).items():
            out.append({key.lower(): val for key, val in project.items()})
            out[-1]["tag"] = tag[1:]
            out[-1]["year"] = out[-1].pop("date")
        toml.dump({"publication": out}, dst)



from typing import Iterator
from contextlib import contextmanager
import tempfile
import os
import sys
import subprocess

import click


@contextmanager
def compile_latex(
    latex: str,
    name: str = "document",
    silent: bool = False,
) -> Iterator[str]:
    """
    Compile latex code.

    Will try to use latexmk first, then pdflatex.
    Assumes that at least one of these executable exists in path.

    Args:
        latex:
            The latex source code as a string.
        name:
            The name of the file to be used during compilation.
            The extensions `.tex` and `.pdf` will be added.
        silent:
            Muffle latex compiles.

    """
    with tempfile.TemporaryDirectory() as folder:
        source_file = os.path.join(folder, f"{name}.tex")

        click.echo(f"writing {source_file}")
        with open(source_file, "w") as dst:
            dst.write(latex)

        sep = "&" if os.name == "nt" else ";"
        silent = "-silent" if silent else ""

        if not silent:
            click.secho(f"trying to compile {name}.tex", fg="yellow")
        proc = subprocess.Popen(
            f'cd "{folder}" {sep} latexmk {source_file} {silent} '
            '-pdf -latexoption="-interaction=nonstopmode"',
            shell=True,
        )
        proc.wait()
        if proc.returncode:
            click.secho("latexmk run failed, see errors above ^^^", fg="red")
            click.secho("trying pdflatex instead...", fg="yellow")
            proc = subprocess.Popen(
                f'cd "{folder}" {sep} pdflatex {source_file} '
                '{silent} -latexoption="-interaction=nonstopmode"',
                shell=True
            )
            proc.wait()
            if proc.returncode:
                click.secho("pdflatex run failed too, see errors above ^^^", fg="red")
                sys.exit(2)
        if not silent:
            click.secho(
                f"Document successfully compiled: {name}.tex -> {name}.pdf", fg="green")
        yield source_file.replace(".tex", ".pdf")



"""Create aggregated values."""
from typing import List
from pydantic import BaseModel, Field

from ..vitae.schema import TechnicalSkill


class SkillCount(BaseModel):
    """Skills and their count."""
    value: str
    count: int


class TopicCount(BaseModel):
    """Field of research and their count."""
    value: str
    count: int


class AggregateContent(BaseModel):
    """Schema for Aggregate content file."""

    technical_skills: List[str]
    topics: List[str]

    num_employees: int = 0
    num_doctors: int = 0
    num_nationalities: int = 0
    num_languages_spoken: int = 0
    num_universities_attended: int = 0

    skill_count: List[SkillCount] = Field(default_factory=list)
    topic_count: List[TopicCount] = Field(default_factory=list)



"""Group skills into categories."""
import os
from typing import Any, Dict, List, Sequence
from collections import defaultdict

import toml

from .schema import TechnicalSkill

CURDIR = os.path.dirname(os.path.abspath(__file__))


def get_skills_data() -> Dict[str, Any]:
    """Get technical skills data."""
    with open(os.path.join(
            CURDIR, os.path.pardir, "data", "tech_skills.toml")) as handler:
        return toml.load(handler)


def make_skill_groups(
    skills: Sequence[str],
    threshold: int = 5,
    cut_below: bool = False,
) -> List[TechnicalSkill]:
    """
    Group skills into categories.

    Prioritizes small groups from unpopular labels to increases the likelihood
    the large and popular labels can take the "leftovers".

    Args:
        skills:
            User provided skills to distribute into groups.
        threshold:
            Prioritize making groups that are at least this size.
            Use `cut_below` to enforce.
        cut_below:
            Remove groups that does not meet the `threshold` criteria.

    Returns:
        Skills distributed over different groups.

    Examples:
        >>> make_skill_groups(  # doctest: +NORMALIZE_WHITESPACE
        ...     ["SQLite", "CVS", "Git", "Pip"], threshold=1)
        [TechnicalSkill(title='Organize', values=['CVS', 'Git', 'Pip']), 
         TechnicalSkill(title='Databases', values=['SQLite'])]
        >>> make_skill_groups(  # doctest: +NORMALIZE_WHITESPACE
        ...     ["SQLite", "CVS", "Git", "Pip"], threshold=2)
        [TechnicalSkill(title='Tools', values=['SQLite']), 
         TechnicalSkill(title='Organize', values=['CVS', 'Git', 'Pip'])]

    """
    skills_data = get_skills_data()

    # get general group label popularity
    count = defaultdict(int)
    for skill in skills_data["skills"]:
        for label in skills_data["skills"][skill]:
            count[label] += 1
    max_count = max(count.values())+1

    skills = list(skills)
    unknown_skills = set(skills).difference(skills_data["skills"])
    assert not unknown_skills, (
            f"unrecognized technical skills: {sorted(unknown_skills)}")

    output = {}
    while skills:

        # make map from group label to skill
        mapping = defaultdict(list)
        for skill in skills:
            for label in skills_data["skills"][skill]:
                mapping[label].append(skill)

        # sort based on primery size of group and secondary general popularity
        keys = sorted(mapping,
                      key=lambda k: max_count*len(mapping[k])+count[k])
        for key in keys:
            if len(mapping[key]) >= threshold:
                break
        else:
            # threshold criteria not met
            if cut_below:
                break

        skills = [skill for skill in skills if skill not in mapping[key]]
        output[key] = sorted(mapping.pop(key))

    output = [TechnicalSkill(title=title, values=output[title])
              for title in skills_data["allowed_labels"] if title in output]
    return output



import os
from typing import Any, List, Sequence

import toml
from .schema import VitaeContent
from .tech_skills import make_skill_groups

CURDIR = f"{os.path.dirname(__file__)}{os.path.sep}"


def filter_(keys: str, sequence: Sequence[Any]) -> List[Any]:
    """
    Filter a sequence form CLI.

    Args:
        keys:
            String with comma-separated tags. Or the string ':'.
        sequence:
            Sequence of elements to filter.

    Returns:
        Same as `sequence`, but filtered down to indices included in `keys`.

    """
    if keys == "":
        return []
    if keys == ":":
        return list(sequence)
    keys = keys.replace(" ", "").split(",")
    return [sequence[idx] for idx, s in enumerate(sequence) if s.tag in keys]


def load_vitae(
    path: str,
    badges: bool = False,
    projects: str = "",
    publications: str = "",
) -> VitaeContent:
    """
    Load TOML content from disk.

    Also filters projects and publication lists, inserts icon prefixes to
    technical skills, and inserts default images as needed.

    Args:
        path:
            Path to the content to load.
        badges:
            Include small badge icons to selected technical skills.
        projects:
            Comma-separated list of project tags to include. ':' includes all.
        publications:
            Comma-separated list of publication tags to include. ':' includes all.

    Returns:
        Loaded content as a nested data structure.

    """
    assert str(path).endswith(".toml"), (
        "must be TOML files with .toml extension.")
    with open(path) as src:
        content = VitaeContent(**toml.load(src))

    # filter projects and publications (as this can not be done in template)
    content.project = filter_(projects, content.project)
    content.publication = filter_(publications, content.publication)

    # place technical skills into groups
    content.technical_skill = make_skill_groups(content.technical_skill)

    if badges:
        for skill in content.technical_skill:
            for idx, value in enumerate(skill.values):
                path = os.path.join(
                    CURDIR, os.path.pardir, "data", "badges", f"{value}.pdf")
                if os.path.isfile(path):
                    skill.values[idx] = (
                        rf"\includegraphics[width=0.3cm]{{{path}}}~{value}")

    # anything with meta.*_image should be an image
    for name in content.meta.__dict__:
        if not name.endswith("_image"):
            continue
        value = getattr(content.meta, name)
        if not os.path.isfile(value):
            setattr(content.meta, name, os.path.join(
                CURDIR, os.path.pardir, "templates", f"{value}.pdf"))
            assert os.path.isfile(getattr(content.meta, name)), (
                f"unrecognized value/path for meta.{name}: '{value}'")

    return content



"""Load latex template engine."""
import os
import glob

from jinja2 import Template

CURDIR = f"{os.path.dirname(__file__)}{os.path.sep}"


def load_template(template_name: str) -> Template:
    """
    Load latex template from disk.

    Template is assumed to be latex code with extra Jinja2 commands:
    `\BLOCK{...}` and `\VAR{...}` for Jinja2 code block and variable insertion.

    Args:
        template_name:
            Either name of a available template, or path to a user provided
            template.

    Returns:
        Template engine for the given template with latex groups.

    """
    template = os.path.join(CURDIR, "templates", template_name)
    if not os.path.isfile(template):
        template = template_name
    assert os.path.isfile(template), (
        f"template '{template}' not valid path")

    with open(template, "r") as src:
        return Template(
            src.read(),
            block_start_string="\\BLOCK{",
            block_end_string="}",
            variable_start_string="\\VAR{",
            variable_end_string="}",
        )



"""Tool for migrating from old txt to new(er) yaml."""
import re
import yaml


def convert_txt_to_yaml(source: str, target: str) -> None:
    """Convert txt format to yaml."""
    data = {
        "Basic": {},
        "Summary": {},
        "Skills": {},
        "Languages": {},
        "SelectedSkills": {},
        "Education": [],
        "Work": [],
        "Interests": {},
        "Projects": {},
        "Publications": {},
    }

    cur = ""

    with open(source, "r") as f:
        text = f.read()

    text = re.sub(r"\n\\\> *", r"\n", text, 0, re.M)
    text = re.sub(r"([^\n]) *\\{2,} *\n", r"\1\n", text, 0, re.M)
    text = re.sub(r" +\n", r"\n", text, 0, re.M)
    text = re.sub(r" +$", r"", text)
    text = re.sub(r"\n{3,}", r"\n\n", text, re.M)

    groups = text.split("\n\n")

    for group in groups:

        header, _, content = group.partition("\n")

        if header == "Name:":
            data["Basic"]["Name"] = content

        elif header == "Address:":
            if "\n" in content:
                address, _, post = content.partition("\n")

                address = re.sub(r",$", "", address)

                data["Basic"]["Address"] = address
                data["Basic"]["Post"] = post

        elif header == "Birth:":
            data["Basic"]["Birth"] = content

        elif header == "Email:":
            data["Basic"]["Email"] = content

        elif header == "Phone:":
            data["Basic"]["Phone"] = content

        elif header == "Summary:":
            content = content.replace("\n", " ")
            data["Summary"] = content

        elif header == "Skills:":

            content = re.sub(r"([^\\]) *\& *", r"\1&", content)
            content = re.sub(r" *, *", r",", content)

            skills = content.split("\n")
            for skill in skills:
                title, _, listicle = skill.partition("&")
                data["Skills"][title] = listicle.split(",")

        elif header == "Languages:":

            content = re.sub(r" *\& *", r"&", content)
            content = content.split("\n")
            for c in content:
                c = c.split("&")
                data["Languages"][c[0]] = c[1]

        elif header == "SelectedSkills:":

            content = re.sub(r"([^\\]) *\& *", r"\1&", content)
            content = re.sub(r" *, *", r",", content)
            content = content.split("\n")
            for c in content:
                title, _, c = c.partition("&")
                data["SelectedSkills"][title] = c

        elif header == "Education:":

            content = re.sub(r"([^\\]) *\& *", r"\1&", content)
            content = content.split("\n")

            for c in content:
                p1, _, p2 = c.partition("&")
                data["Education"].append([p1, p2])

        elif header == "Work:":

            content = re.sub(r" *\& *", r"&", content)
            content = re.sub(r" *-- *", r"--", content)
            content = content.split("\n")

            for c in content:
                title, _, c = c.partition("&")
                if "--" in title:
                    t1, _, t2 = title.partition("--")
                    data["Work"].append([t1, t2, c])
                else:
                    data["Work"].append([title, c])

        elif header == "Interests:":

            content = re.sub(r" *\& *", r"&", content)
            content = re.sub(r" *, *", r",", content)

            skills = content.split("\n")
            for skill in skills:
                title, _, listicle = skill.partition("&")
                data["Interests"][title] = listicle.split(",")

        elif header[0] == "A":

            header = "A%d" % (int(header[1:-1])+1)
            data["Projects"][header] = a = {}

            for line in content.split("\n"):
                line = re.sub(r"^(\w+): +", r"\1:", line, 0, re.M)

                if line[:9] == "Activity:":
                    cur = "Activity"
                    a["Activity"] = line[9:]

                elif line[:5] == "Role:":
                    cur = "Role"
                    a["Role"] = line[5:]

                elif line[:9] == "Staffing:":
                    cur = "Staffing"
                    a["Staffing"] = line[9:]

                elif line[:12] == "Description:":
                    cur = "Description"
                    a["Description"] = line[12:]

                elif line[:6] == "Tools:":
                    cur = "Tools"
                    a["Tools"] = line[6:]
                else:
                    a[cur] += " " + line

        elif header[0] == "B":

            header = "B%d" % (int(header[1:-1])+1)
            data["Publications"][header] = b = {}

            entries = ["Journal", "Date", "Authors", "Title", "Summary", "DOI"]
            for entry in entries:
                if line.startswith(entry):
                    cur = entry
                    b[entry] = line[len(entry):]
                    break
            else:
                a[cur] += " " + line

    txt = yaml.dump(data, default_flow_style=False)

    target = target or source.replace(".txt", ".yaml")
    with open(target, "w") as f:
        f.write(txt)



"""Command line interface parser."""
from typing import List
import os
import re
import shutil
import glob

import toml
import click
from click_help_colors import HelpColorsGroup, HelpColorsCommand

from .compile import compile_latex
from .template import load_template

from .vitae import load_vitae
from .aggregate import load_aggregate
from .txt2yaml import convert_txt_to_yaml
from .yaml2toml import convert_yaml_to_toml


@click.group(
    cls=HelpColorsGroup,
    help_headers_color="green",
    help_options_color="blue",
    short_help="CV creator tool",
)
def cv():
    """
    Command line tool for creating curriculum vitae from TOML source files.
    """


@cv.command(
    cls=HelpColorsCommand,
    help_options_color="blue",
    short_help="Create CV as .pdf file",
)
@click.argument("toml_content")
@click.argument("output", default="")
@click.option("-b", "--badges", is_flag=True, help=(
    "Include small badge icons to selected technical skills."))
@click.option("-l", "--latex", is_flag=True, help=(
    "Output latex document instead of a pdf."))
@click.option("-p", "--projects", default="", help=(
    "Comma-separated list of project tags to include. Use ':' for all."))
@click.option("-u", "--publications", default="", help=(
    "Comma-separated list of publication tags to include. Use ':' for all."))
def create(
    toml_content: str,
    output: str = "",
    badges: bool = False,
    latex: bool = False,
    projects: str = "",
    publications: str = "",
) -> None:
    """
    Create curriculum vitae from TOML content file.
    """
    content = load_vitae(toml_content, badges=badges,
                         projects=projects, publications=publications)
    template = load_template("vitae.tex")
    latex_code = template.render(**dict(content))
    if latex:
        output = output or toml_content.replace(".toml", ".tex")
        with open(output, "w") as dst:
            dst.write(latex_code)
    else:
        name = os.path.basename(toml_content.replace(".toml", ""))
        output = output or toml_content.replace(".toml", ".pdf")
        with compile_latex(latex=latex_code, name=name) as pdf_path:
            shutil.copy(pdf_path, output)


@cv.command(cls=HelpColorsCommand, short_help="Create aggregate CSV file")
@click.argument("agg_content")
@click.argument("cv_content", nargs=-1, required=True)
def aggregate(
    agg_content: str,
    cv_content: List[str],
):
    """
    Create aggregate .csv from series of .toml content files.
    """
    content = load_aggregate(agg_content, cv_content)
    template = load_template("aggregate.csv")
    csv_string = template.render(**dict(content))
    output = agg_content.replace(".toml", ".csv")
    with open(output, "w") as dst:
        dst.write(csv_string)


@cv.command(cls=HelpColorsCommand, short_help="Convert old .txt to .yaml")
@click.argument("txt_source")
@click.argument("yaml_target", default="")
def txt2yaml(txt_source, yaml_target):
    """
    Convert old .txt content file into newer(-ish) .yaml format.
    """
    convert_txt_to_yaml(txt_source, yaml_target)


@cv.command(cls=HelpColorsCommand, short_help="Convert old .yaml to .toml")
@click.argument("yaml_source")
@click.argument("toml_target", default="")
def yaml2toml(yaml_source, toml_target):
    """
    Convert old .yaml content file into newer .toml format.
    """
    convert_yaml_to_toml(yaml_source, toml_target)


@cv.command(cls=HelpColorsCommand, short_help="Create example TOML file")
@click.argument("toml_target")
def example(toml_target):
    """
    Create example .toml content file to be filled out.
    """
    toml_source = os.path.join(os.path.dirname(__file__), "templates", "example.toml")
    shutil.copy(toml_source, toml_target)


@cv.command(cls=HelpColorsCommand, short_help="List allowed technical skills")
@click.option("--badges", is_flag=True, help=(
    "Only list skills that has an associated icon."))
def skills(badges):
    """
    List of allowed technical skills. The spelling is case sensitive.
    """
    if badges:
        icons = glob.glob(os.path.join(os.path.dirname(__file__), "data", "badges", "*.pdf"))
        data = (re.sub(r".*/([^/]+).pdf$", r"\1", icon) for icon in icons)
    else:
        path = os.path.join(os.path.dirname(__file__), "data", "tech_skills.toml")
        with open(path) as handle:
            data = toml.load(handle)["skills"]
    click.echo("\n".join(data))



# pylint: disable=too-few-public-methods
"""Schema definition for the user provided yaml source file."""
import datetime
from typing import List, Literal, Optional, Union

from pydantic import BaseModel, Field
import pycountry


COUNTRIES = tuple(country.__dict__.get(
    "common_name", country.name).replace(", Islamic Republic of", "")
    for country in pycountry.countries)
Country = Literal[COUNTRIES]

LANGUAGES = tuple(language.name for language in pycountry.languages)
Language = Literal[LANGUAGES]

# hopefully exhaustive:
NATIONALITIES = (
    'Afghan', 'Albanian', 'Algerian', 'American', 'Andorran', 'Angolan',
    'Antiguans', 'Argentinean', 'Armenian', 'Australian', 'Austrian',
    'Azerbaijani', 'Bahamian', 'Bahraini', 'Bangladeshi', 'Barbadian',
    'Barbudans', 'Batswana', 'Belarusian', 'Belgian', 'Belizean', 'Beninese',
    'Bhutanese', 'Bolivian', 'Bosnian', 'Brazilian', 'British', 'Bruneian',
    'Bulgarian', 'Burkinabe', 'Burmese', 'Burundian', 'Cambodian',
    'Cameroonian', 'Canadian', 'Cape Verdean', 'Central African', 'Chadian',
    'Chilean', 'Chinese', 'Colombian', 'Comoran',  'Congolese', 'Costa Rican',
    'Croatian', 'Cuban', 'Cypriot', 'Czech', 'Danish', 'Djibouti', 'Dominican',
    'Dutch', 'Dutchman', 'Dutchwoman', 'East Timorese', 'Ecuadorean',
    'Egyptian', 'Emirian', 'Equatorial Guinean', 'Eritrean', 'Estonian',
    'Ethiopian', 'Fijian', 'Filipino', 'Finnish', 'French', 'Gabonese',
    'Gambian', 'Georgian', 'German', 'Ghanaian', 'Greek', 'Grenadian',
    'Guatemalan', 'Guinea-Bissauan', 'Guinean', 'Guyanese', 'Haitian',
    'Herzegovinian', 'Honduran', 'Hungarian', 'I-Kiribati', 'Icelander',
    'Indian', 'Indonesian', 'Iranian', 'Iraqi', 'Irish', 'Israeli', 'Italian',
    'Ivorian', 'Jamaican', 'Japanese', 'Jordanian', 'Kazakhstani', 'Kenyan',
    'Kittian and Nevisian', 'Kuwaiti', 'Kyrgyz', 'Laotian', 'Latvian',
    'Lebanese', 'Liberian', 'Libyan', 'Liechtensteiner', 'Lithuanian',
    'Luxembourger', 'Macedonian', 'Malagasy', 'Malawian', 'Malaysian',
    'Maldivan', 'Malian', 'Maltese', 'Marshallese', 'Mauritanian', 'Mauritian',
    'Mexican', 'Micronesian', 'Moldovan', 'Monacan', 'Mongolian', 'Moroccan',
    'Mosotho', 'Motswana', 'Mozambican', 'Namibian', 'Nauruan', 'Nepalese',
    'Netherlander', 'New Zealander', 'Ni-Vanuatu', 'Nicaraguan', 'Nigerian',
    'Nigerien', 'North Korean', 'Northern Irish', 'Norwegian', 'Omani',
    'Pakistani', 'Palauan', 'Panamanian', 'Papua New Guinean', 'Paraguayan',
    'Peruvian', 'Polish', 'Portuguese', 'Qatari', 'Romanian', 'Russian',
    'Rwandan', 'Saint Lucian', 'Salvadoran', 'Samoan', 'San Marinese',
    'Sao Tomean', 'Saudi', 'Scottish', 'Senegalese', 'Serbian', 'Seychellois',
    'Sierra Leonean', 'Singaporean', 'Slovakian', 'Slovenian',
    'Solomon Islander', 'Somali', 'South African', 'South Korean', 'Spanish',
    'Sri Lankan', 'Sudanese', 'Surinamer', 'Swazi', 'Swedish', 'Swiss',
    'Syrian', 'Taiwanese', 'Tajik', 'Tanzanian', 'Thai', 'Togolese', 'Tongan',
    'Trinidadian or Tobagonian', 'Tunisian', 'Turkish', 'Tuvaluan', 'Ugandan',
    'Ukrainian', 'Uruguayan', 'Uzbekistani', 'Venezuelan', 'Vietnamese',
    'Welsh', 'Yemenite', 'Zambian', 'Zimbabwean'
)
Nationality = Literal[NATIONALITIES]


class StrictModel(BaseModel):
    """Same as baseclass, but forbit superfluous variables."""

    class Config:
        extra = "forbid"


class TechnicalSkill(StrictModel):
    """Group of technical skills."""

    title: str
    values: List[str]


class LanguageSkill(StrictModel):
    """Language skill and proficiency."""

    language: Language
    proficiency: Literal["Native", "Fluent", "Intermediate", "Basic"]


class PersonalSkill(StrictModel):
    """A personal skill and description."""

    title: str
    description: str


class Hobby(StrictModel):
    """Group of hobbies."""

    title: str
    values: List[str]


class Education(StrictModel):
    """Completed educational degree."""

    start: int = 0
    end: int = 0
    degree: Literal["Bachelor's degree", "Master's degree", "PhD",
                    "Diploma degree", "Cand. Scient", "Doctor Scient", ""] = ""
    topic: Literal["Physics", "Scientific Computing", "Mechanics",
                   "Mathematics", "Engineering", "Chemestry",
                   "Geology and Geophysics", "Computer Science", ""] = ""
    specialization: str = ""
    thesis_title: str = ""
    department: str = ""
    university: str = ""
    country: Country = ""
    description: str = ""


class Work(StrictModel):
    """Previous work experience."""

    start: str
    end: str = ""
    description: str


class Project(StrictModel):
    """Extended description of a project."""

    activity: str
    role: str = ""
    staffing: str = ""
    period: str = ""
    description: str
    tools: str = ""
    volume: str = ""
    url: str = ""
    tag: str = ""


class Publications(StrictModel):
    """Published journal papers."""

    journal: str
    title: str
    doi: str
    authors: str
    year: int
    tag: str = ""
    description: str = ""


class MetaInformation(StrictModel):
    """Meta-information used by the document."""

    font_size: int = 11
    logo_image: str = "logo"
    footer_image: str = "footer"
    email_image: str = "email"
    address_image: str = "address"
    github_image: str = "link"
    website_image: str = "link"
    phone_image: str = "phone"
    birth_image: str = "birth"
    nationality_image: str = "nationality"


class VitaeContent(StrictModel):
    """Schema for Vitae content file."""

    name: str
    address: str = ""
    post: str = ""
    birth: Optional[datetime.date] = None
    email: str = ""
    phone: str = ""
    nationality: Optional[Nationality] = None
    github: str = ""
    website: str = ""
    summary: str = ""

    meta: MetaInformation = MetaInformation()

    # Should be TechnicalSkill, but is constructed after parsing.
    # 'str' is used here as a placeholder for list of skills.
    technical_skill: Union[List[str], List[TechnicalSkill]] = (
        Field(default_factory=list))

    language_skill: List[LanguageSkill] = Field(default_factory=list)
    personal_skill: List[PersonalSkill] = Field(default_factory=list)
    hobby: List[Hobby] = Field(default_factory=list)
    education: List[Education] = Field(default_factory=list)
    work: List[Work] = Field(default_factory=list)
    project: List[Project] = Field(default_factory=list)
    publication: List[Publications] = Field(default_factory=list)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cvcreator', 'cvcreator.aggregate', 'cvcreator.vitae']

package_data = \
{'': ['*'], 'cvcreator': ['data/*', 'data/badges/*', 'templates/*']}

install_requires = \
['Jinja2>=2.11.3,<3.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'click-help-colors>=0.9,<0.10',
 'click>=8.0.1,<9.0.0',
 'pycountry>=20.7.3,<21.0.0',
 'pydantic>=1.8.1,<2.0.0',
 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['cv = cvcreator.parser:cv',
                     'cvcreate = cvcreator.parser:create']}

setup_kwargs = {
    'name': 'cvcreator',
    'version': '1.1.0',
    'description': 'An automated tool for creating CVs on the fly.',
    'long_description': None,
    'author': 'Jonathan Feinberg',
    'author_email': 'jonathf@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)
